==> tib-0.7.0.tgz: package/lib/tib-browser.js <==
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

const path$1 = _interopDefault(require('path'));
const Hookable = _interopDefault(require('hable'));
const onExit = _interopDefault(require('signal-exit'));
const os = _interopDefault(require('os'));
const child_process = require('child_process');
const kill = _interopDefault(require('tree-kill'));
const browserstackLocal = require('./tib-browserstack-local.js');
const http = _interopDefault(require('http'));

const consola = console; // eslint-disable-line no-console

const supportedPlatforms = [
  'linux',
  'freebsd',
  'openbsd'
];

class Xvfb {
  static load(browser) {
    if (!browser.config.xvfb) {
      return
    }

    Xvfb.isSupported(true);

    const browserConfig = browser.config.browserConfig;
    if (browserConfig && browserConfig.window) {
      if (typeof browser.config.xvfb !== 'object') {
        browser.config.xvfb = { args: [] };
      }

      browser.config.xvfb.args.push(`-screen 0 ${browserConfig.window.width}x${browserConfig.window.height}x24`);
    }

    const config = {
      quiet: browser.config.quiet,
      ...browser.config.xvfb
    };

    browser.hook('start:before', () => Xvfb.start(config));
    browser.hook('close:after', Xvfb.stop);
  }

  static isSupported(failWhenNot) {
    const platform = os.platform();
    const supported = supportedPlatforms.includes(platform);

    if (!supported && failWhenNot) {
      throw new browserstackLocal.BrowserError(`Xvfb is not supported on ${platform} platforms`)
    }

    return supported
  }

  static isRunning() {
    return !!Xvfb.process && Xvfb.process.connected && !Xvfb.closed
  }

  static start({ displayNum = 99, args = [], quiet } = {}) {
    Xvfb.isSupported(true);
    Xvfb.closed = false;

    if (Xvfb.isRunning()) {
      return
    }

    args = (Array.isArray(args) && args) || [];

    const display = `:${displayNum}`;
    args.unshift(display);
    args.unshift('Xvfb');

    Xvfb.process = child_process.spawn('command', args, {
      shell: true,
      stdio: [
        'ignore',
        'ignore',
        'pipe'
      ]
    });

    // set DISPLAY env
    process.env.DISPLAY = display;

    let stderr = '';
    Xvfb.process.stderr.on('data', data => (stderr += `${data}`));

    Xvfb.process.on('error', (err) => {
      Xvfb.closed = true;

      if (err && err.code === 'ENOENT') {
        throw new browserstackLocal.BrowserError(`Xvfb not found, please make sure Xvfb is installed`)
      }
    });

    Xvfb.process.on('close', (code, signal) => {
      Xvfb.closed = true;

      if (code === 1) {
        const error = stderr.match(/\(EE\) (?!\(EE\))(.+?)$/m)[1] || stderr;
        if (stderr.includes('already active for display')) {
          if (!quiet) {
            console.warn(`Xvfb: ${error}`); // eslint-disable-line no-console
          }
          return
        }

        throw new browserstackLocal.BrowserError(`Failed to start Xvfb${error ? ', ' : ''}${error}`)
      }
    });

    onExit(() => Xvfb.stop());
  }

  static stop() {
    if (!Xvfb.process || Xvfb.closed) {
      return
    }

    kill(Xvfb.process.pid);

    // enable timers if they where faked by a test framework
    browserstackLocal.enableTimers();

    const waitTimeout = new Promise((resolve) => {
      const timeout = setTimeout(() => {
        consola.warn(`Timeout: Xvfb did not exit after 3s`);
        resolve();
      }, 3000);
      timeout.unref();
    });

    const waitClosed = new Promise((resolve) => {
      const closeInterval = setInterval(() => {
        if (Xvfb.closed) {
          clearInterval(closeInterval);
          resolve();
        }
      }, 50);
    });

    return Promise.race([waitClosed, waitTimeout])
  }
}

class StaticServer {
  static async loadDependencies() {
    if (StaticServer.serveStatic) {
      return
    }

    StaticServer.serveStatic = await browserstackLocal.loadDependency('serve-static');
    StaticServer.finalhandler = await browserstackLocal.loadDependency('finalhandler');
  }

  static load(browser) {
    if (!browser.config.staticServer) {
      return
    }

    if (browser.config.staticServer === true) {
      const { folder } = browser.config;
      browser.config.staticServer = { folder };
    }

    if (!browser.config.staticServer.folder) {
      return
    }

    browser.hook('start:before', () => StaticServer.start(browser.config.staticServer, browser.config.quiet));
    browser.hook('close:after', StaticServer.stop);
  }

  static async start(config, quiet) {
    await StaticServer.loadDependencies();

    const host = process.env.HOST || config.host || 'localhost';
    const port = process.env.PORT || config.port || 3000;

    const serve = StaticServer.serveStatic(config.folder);

    const server = http.createServer((req, res) => {
      serve(req, res, StaticServer.finalhandler(req, res));
    });

    await new Promise((resolve, reject) => {
      server.on('error', reject);
      server.listen(port, host, () => {
        if (!quiet) {
          // eslint-disable-next-line no-console
          console.info(`tib: Static server started on http://${host}:${port}`);
        }

        config.host = host;
        config.port = port;

        StaticServer.server = server;

        resolve(server);
      });
    });
  }

  static stop() {
    if (StaticServer.server) {
      return new Promise((resolve) => {
        StaticServer.server.close(() => {
          StaticServer.server = undefined;
          resolve();
        });
      })
    }
  }
}



const index = /*#__PURE__*/Object.freeze({
  Xvfb: Xvfb,
  BrowserStackLocal: browserstackLocal.BrowserStackLocal,
  StaticServer: StaticServer
});

/**
 * THIS FILE IS AUTOMATICALLY GENERATED
 * DONT CHANGE ANYTHING MANUALLY
 */

const _interopDefault$1 = i => i.then(m => m.default || m);

const browsers = {
  'browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index2.js')); })),
  'browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local.js')); })),
  'chrome': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index3.js')); })),
  'chrome/browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index4.js')); })),
  'chrome/browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local2.js')); })),
  'chrome/puppeteer': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-puppeteer.js')); })),
  'chrome/selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-selenium.js')); })),
  'edge': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index6.js')); })),
  'edge/browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index7.js')); })),
  'edge/browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local3.js')); })),
  'edge/selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-selenium2.js')); })),
  'firefox': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index8.js')); })),
  'firefox/browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index9.js')); })),
  'firefox/browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local4.js')); })),
  'firefox/selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-selenium3.js')); })),
  'ie': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index10.js')); })),
  'ie/browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index11.js')); })),
  'ie/browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local5.js')); })),
  'ie/selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-selenium4.js')); })),
  'jsdom': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index12.js')); })),
  'puppeteer': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index5.js')); })),
  'puppeteer/core': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-core.js')); })),
  'safari': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index13.js')); })),
  'safari/browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index14.js')); })),
  'safari/browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local6.js')); })),
  'safari/selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-selenium5.js')); })),
  'saucelabs': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index15.js')); })),
  'selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index.js')); }))
};

class Browser extends Hookable {
  constructor(config = {}) {
    super();

    browserstackLocal.abstractGuard('Browser', new.target);

    this.config = {
      quiet: false,
      ...config
    };

    this.ready = false;

    if (config.extendPage && typeof config.extendPage === 'function') {
      this.hook('page:after', async (page) => {
        const extendWith = await config.extendPage(page);
        if (extendWith && typeof extendWith === 'object') {
          page.extend(extendWith);
        }
      });
    }

    this.capabilities = {};

    // before browserConfig
    this.config.browserArguments = this.config.browserArguments || [];

    if (this.config.browserConfig) {
      for (const key in this.config.browserConfig) {
        if (key.startsWith('driver') || key.startsWith('provider') || key === 'browserVariant') {
          continue
        }

        if (key === 'staticserver') {
          if (!this.config.staticServer) {
            this.config.staticServer = true;
          }
          continue
        }

        if (key === 'xvfb') {
          if (this.config.browserConfig[key] === 'false' || this.config.browserConfig.headless) {
            continue
          }

          if (!this.config.xvfb) {
            this.config.xvfb = Xvfb.isSupported();
          }

          continue
        }

        const fn = `set${key.charAt(0).toUpperCase()}${key.slice(1)}`;
        if (this[fn]) {
          this[fn](this.config.browserConfig[key]);
        } else {
          console.warn(`browserConfig '${key}' could not be set`); // eslint-disable-line no-console
        }
      }
    }

    if (!this.config.xvfb && this.config.xvfb !== false) {
      this.config.xvfb = Xvfb.isSupported();
    }

    Xvfb.load(this);
    StaticServer.load(this);

    if (browserstackLocal.isMockedFunction(setTimeout, 'setTimeout')) {
      // eslint-disable-next-line no-console
      console.warn(`Mocked timers detected

The browser probably won't ever start with globally mocked timers. Will try to automatically use real timers on start and set to use fake timers after start. If the browser still hangs and doesn't start, make sure to only mock the global timers after the browser has started `);

      this.hook('start:before', () => browserstackLocal.enableTimers());
      this.hook('start:after', () => browserstackLocal.disableTimers());
    }
  }

  static async get(browserString = '', config = {}) {
    const browserConfig = browserstackLocal.getBrowserConfigFromString(browserString);
    const browserImport = browserstackLocal.getBrowserImportFromConfig(browserConfig);

    if (!browsers[browserImport]) {
      throw new browserstackLocal.BrowserError(`Unknown browser, no import exists for '${browserImport}'`)
    }

    try {
      // add browserConfig to config
      config.browserConfig = browserConfig;

      const Browser = await browsers[browserImport]();

      const browserInstance = new Browser(config);
      await browserInstance.loadDependencies();
      return browserInstance
    } catch (e) {
      if (e instanceof browserstackLocal.BrowserError) {
        throw e
      } else {
        throw new browserstackLocal.BrowserError(`Error occured while loading '${browserConfig.browser || browserString}' browser`, e)
      }
    }
  }

  setLogLevel(level) {}

  async loadDependency(dependency) {
    try {
      return await browserstackLocal.loadDependency(dependency)
    } catch (e) {
      throw new browserstackLocal.BrowserError(this, e.message)
    }
  }

  _loadDependencies() {}

  async loadDependencies(...args) {
    await this.callHook('dependencies:load');

    await this._loadDependencies(...args);

    await this.callHook('dependencies:loaded');
  }

  getUrl(urlPath) {
    if (this.config.staticServer) {
      const { host, port } = this.config.staticServer;
      return `http://${host}:${port}${urlPath}`
    }

    return `file://${path$1.join(this.config.folder, urlPath)}`
  }

  getCapabilities(capabilities) {
    if (!capabilities) {
      return this.capabilities
    }

    return {
      ...this.capabilities,
      ...capabilities
    }
  }

  getCapability(capability) {
    return this.capabilities[capability]
  }

  addCapability(key, value) {
    this.capabilities[key] = value;
    return this
  }

  addCapabilities(capabilities) {
    this.capabilities = {
      ...this.capabilities,
      ...capabilities
    };
    return this
  }

  setWindow(width, height) {
    if (!height && typeof width === 'object') {
      this.config.window = width;
      return this
    }

    this.config.window = { width, height };
    return this
  }

  getBrowser(name) {
    return this.getCapability('browserName')
  }

  setBrowser(name, version = '') {
    this.addCapability('browserName', name);

    if (version) {
      this.setBrowserVersion(version);
    }

    return this
  }

  setHeadless() {
    this.config.xvfb = false;
    return this
  }

  getBrowserVersion() { return undefined }

  setBrowserVersion() { return this }

  setOs(...args) { return this.setOS(...args) }

  setOsVersion(...args) { return this.setOSVersion(...args) }

  setOS() { return this }

  setOSVersion() { return this }

  setDevice() { return this }

  isReady() {
    return this.ready
  }

  _start() {}

  _close() {}

  _page() {}

  async start(capabilities, ...args) {
    await this.callHook('start:before');

    try {
      await this._start(capabilities, ...args);

      await this.callHook('start:after', this.driver);

      this.ready = true;

      onExit(() => this.close());

      return this
    /* istanbul ignore next */
    } catch (e) {
      await this.close();

      throw new browserstackLocal.BrowserError(e)
    }
  }

  async close(...args) {
    await this.callHook('close:before');

    await this._close(...args);

    await this.callHook('close:after');
  }

  async page(...args) {
    await this.callHook('page:before');

    const page = await this._page(...args);

    await this.callHook('page:after', page);

    return page
  }
}

exports.Browser = Browser;
exports.index = index;

==> tib-0.7.1.tgz: package/lib/tib-browser.js <==
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

const path = _interopDefault(require('path'));
const Hookable = _interopDefault(require('hable'));
const onExit = _interopDefault(require('signal-exit'));
const os = _interopDefault(require('os'));
const child_process = require('child_process');
const kill = _interopDefault(require('tree-kill'));
const browserstackLocal = require('./tib-browserstack-local.js');
const http = _interopDefault(require('http'));

const consola = console; // eslint-disable-line no-console

const supportedPlatforms = [
  'linux',
  'freebsd',
  'openbsd'
];

class Xvfb {
  static load(browser) {
    if (!browser.config.xvfb) {
      return
    }

    Xvfb.isSupported(true);

    const browserConfig = browser.config.browserConfig;
    if (browserConfig && browserConfig.window) {
      if (typeof browser.config.xvfb !== 'object') {
        browser.config.xvfb = { args: [] };
      }

      browser.config.xvfb.args.push(`-screen 0 ${browserConfig.window.width}x${browserConfig.window.height}x24`);
    }

    const config = {
      quiet: browser.config.quiet,
      ...browser.config.xvfb
    };

    browser.hook('start:before', () => Xvfb.start(config));
    browser.hook('close:after', Xvfb.stop);
  }

  static isSupported(failWhenNot) {
    const platform = os.platform();
    const supported = supportedPlatforms.includes(platform);

    if (!supported && failWhenNot) {
      throw new browserstackLocal.BrowserError(`Xvfb is not supported on ${platform} platforms`)
    }

    return supported
  }

  static isRunning() {
    return !!Xvfb.process && Xvfb.process.connected && !Xvfb.closed
  }

  static start({ displayNum = 99, args = [], quiet } = {}) {
    Xvfb.isSupported(true);
    Xvfb.closed = false;

    if (Xvfb.isRunning()) {
      return
    }

    args = (Array.isArray(args) && args) || [];

    const display = `:${displayNum}`;
    args.unshift(display);
    args.unshift('Xvfb');

    Xvfb.process = child_process.spawn('command', args, {
      shell: true,
      stdio: [
        'ignore',
        'ignore',
        'pipe'
      ]
    });

    // set DISPLAY env
    process.env.DISPLAY = display;

    let stderr = '';
    Xvfb.process.stderr.on('data', data => (stderr += `${data}`));

    Xvfb.process.on('error', (err) => {
      Xvfb.closed = true;

      if (err && err.code === 'ENOENT') {
        throw new browserstackLocal.BrowserError(`Xvfb not found, please make sure Xvfb is installed`)
      }
    });

    Xvfb.process.on('close', (code, signal) => {
      Xvfb.closed = true;

      if (code === 1) {
        const error = stderr.match(/\(EE\) (?!\(EE\))(.+?)$/m)[1] || stderr;
        if (stderr.includes('already active for display')) {
          if (!quiet) {
            console.warn(`Xvfb: ${error}`); // eslint-disable-line no-console
          }
          return
        }

        throw new browserstackLocal.BrowserError(`Failed to start Xvfb${error ? ', ' : ''}${error}`)
      }
    });

    onExit(() => Xvfb.stop());
  }

  static stop() {
    if (!Xvfb.process || Xvfb.closed) {
      return
    }

    kill(Xvfb.process.pid);

    // enable timers if they where faked by a test framework
    browserstackLocal.enableTimers();

    const waitTimeout = new Promise((resolve) => {
      const timeout = setTimeout(() => {
        consola.warn(`Timeout: Xvfb did not exit after 3s`);
        resolve();
      }, 3000);
      timeout.unref();
    });

    const waitClosed = new Promise((resolve) => {
      const closeInterval = setInterval(() => {
        if (Xvfb.closed) {
          clearInterval(closeInterval);
          resolve();
        }
      }, 50);
    });

    return Promise.race([waitClosed, waitTimeout])
  }
}

class StaticServer {
  static async loadDependencies() {
    if (StaticServer.serveStatic) {
      return
    }

    StaticServer.serveStatic = await browserstackLocal.loadDependency('serve-static');
    StaticServer.finalhandler = await browserstackLocal.loadDependency('finalhandler');
  }

  static load(browser) {
    if (!browser.config.staticServer) {
      return
    }

    if (browser.config.staticServer === true) {
      const { folder } = browser.config;
      browser.config.staticServer = { folder };
    }

    if (!browser.config.staticServer.folder) {
      return
    }

    browser.hook('start:before', () => StaticServer.start(browser.config.staticServer, browser.config.quiet));
    browser.hook('close:after', StaticServer.stop);
  }

  static async start(config, quiet) {
    await StaticServer.loadDependencies();

    const host = process.env.HOST || config.host || 'localhost';
    const port = process.env.PORT || config.port || 3000;

    const serve = StaticServer.serveStatic(config.folder);

    const server = http.createServer((req, res) => {
      serve(req, res, StaticServer.finalhandler(req, res));
    });

    await new Promise((resolve, reject) => {
      server.on('error', reject);
      server.listen(port, host, () => {
        if (!quiet) {
          // eslint-disable-next-line no-console
          console.info(`tib: Static server started on http://${host}:${port}`);
        }

        config.host = host;
        config.port = port;

        StaticServer.server = server;

        resolve(server);
      });
    });
  }

  static stop() {
    if (StaticServer.server) {
      return new Promise((resolve) => {
        StaticServer.server.close(() => {
          StaticServer.server = undefined;
          resolve();
        });
      })
    }
  }
}



const index = /*#__PURE__*/Object.freeze({
  Xvfb: Xvfb,
  BrowserStackLocal: browserstackLocal.BrowserStackLocal,
  StaticServer: StaticServer
});

/**
 * THIS FILE IS AUTOMATICALLY GENERATED
 * DONT CHANGE ANYTHING MANUALLY
 */

const _interopDefault$1 = i => i.then(m => m.default || m);

const browsers = {
  'browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index2.js')); })),
  'browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local.js')); })),
  'chrome': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index3.js')); })),
  'chrome/browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index4.js')); })),
  'chrome/browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local2.js')); })),
  'chrome/puppeteer': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-puppeteer.js')); })),
  'chrome/selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-selenium.js')); })),
  'edge': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index6.js')); })),
  'edge/browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index7.js')); })),
  'edge/browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local3.js')); })),
  'edge/selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-selenium2.js')); })),
  'firefox': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index8.js')); })),
  'firefox/browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index9.js')); })),
  'firefox/browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local4.js')); })),
  'firefox/selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-selenium3.js')); })),
  'ie': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index10.js')); })),
  'ie/browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index11.js')); })),
  'ie/browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local5.js')); })),
  'ie/selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-selenium4.js')); })),
  'jsdom': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index12.js')); })),
  'puppeteer': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index5.js')); })),
  'puppeteer/core': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-core.js')); })),
  'safari': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index13.js')); })),
  'safari/browserstack': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index14.js')); })),
  'safari/browserstack/local': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-local6.js')); })),
  'safari/selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-selenium5.js')); })),
  'saucelabs': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index15.js')); })),
  'selenium': () => _interopDefault$1(new Promise(function (resolve) { resolve(require('./tib-index.js')); }))
};

class Browser extends Hookable {
  constructor(config = {}) {
    super();

    browserstackLocal.abstractGuard('Browser', new.target);

    this.config = {
      quiet: false,
      ...config
    };

    this.ready = false;

    if (config.extendPage && typeof config.extendPage === 'function') {
      this.hook('page:after', async (page) => {
        const extendWith = await config.extendPage(page);
        if (extendWith && typeof extendWith === 'object') {
          page.extend(extendWith);
        }
      });
    }

    this.capabilities = {};

    // before browserConfig
    this.config.browserArguments = this.config.browserArguments || [];

    if (this.config.browserConfig) {
      for (const key in this.config.browserConfig) {
        if (key.startsWith('driver') || key.startsWith('provider') || key === 'browserVariant') {
          continue
        }

        if (key === 'staticserver') {
          if (!this.config.staticServer) {
            this.config.staticServer = true;
          }
          continue
        }

        if (key === 'xvfb') {
          if (this.config.browserConfig[key] === 'false' || this.config.browserConfig.headless) {
            continue
          }

          if (!this.config.xvfb) {
            this.config.xvfb = Xvfb.isSupported();
          }

          continue
        }

        const fn = `set${key.charAt(0).toUpperCase()}${key.slice(1)}`;
        if (this[fn]) {
          this[fn](this.config.browserConfig[key]);
        } else {
          console.warn(`browserConfig '${key}' could not be set`); // eslint-disable-line no-console
        }
      }
    }

    if (!this.config.xvfb && this.config.xvfb !== false) {
      this.config.xvfb = Xvfb.isSupported();
    }

    Xvfb.load(this);
    StaticServer.load(this);

    if (browserstackLocal.isMockedFunction(setTimeout, 'setTimeout')) {
      // eslint-disable-next-line no-console
      console.warn(`Mocked timers detected

The browser probably won't ever start with globally mocked timers. Will try to automatically use real timers on start and set to use fake timers after start. If the browser still hangs and doesn't start, make sure to only mock the global timers after the browser has started `);

      this.hook('start:before', () => browserstackLocal.enableTimers());
      this.hook('start:after', () => browserstackLocal.disableTimers());
    }
  }

  static async get(browserString = '', config = {}) {
    const browserConfig = browserstackLocal.getBrowserConfigFromString(browserString);
    const browserImport = browserstackLocal.getBrowserImportFromConfig(browserConfig);

    if (!browsers[browserImport]) {
      throw new browserstackLocal.BrowserError(`Unknown browser, no import exists for '${browserImport}'`)
    }

    try {
      // add browserConfig to config
      config.browserConfig = browserConfig;

      const Browser = await browsers[browserImport]();

      const browserInstance = new Browser(config);
      await browserInstance.loadDependencies();
      return browserInstance
    } catch (e) {
      if (e instanceof browserstackLocal.BrowserError) {
        throw e
      } else {
        throw new browserstackLocal.BrowserError(`Error occured while loading '${browserConfig.browser || browserString}' browser`, e)
      }
    }
  }

  setLogLevel(level) {}

  async loadDependency(dependency) {
    try {
      return await browserstackLocal.loadDependency(dependency)
    } catch (e) {
      throw new browserstackLocal.BrowserError(this, e.message)
    }
  }

  _loadDependencies() {}

  async loadDependencies(...args) {
    await this.callHook('dependencies:load');

    await this._loadDependencies(...args);

    await this.callHook('dependencies:loaded');
  }

  getUrl(urlPath) {
    if (this.config.staticServer) {
      const { host, port } = this.config.staticServer;
      return `http://${host}:${port}${urlPath}`
    }

    return `file://${path.join(this.config.folder, urlPath)}`
  }

  getCapabilities(capabilities) {
    if (!capabilities) {
      return this.capabilities
    }

    return {
      ...this.capabilities,
      ...capabilities
    }
  }

  getCapability(capability) {
    return this.capabilities[capability]
  }

  addCapability(key, value) {
    this.capabilities[key] = value;
    return this
  }

  addCapabilities(capabilities) {
    this.capabilities = {
      ...this.capabilities,
      ...capabilities
    };
    return this
  }

  setWindow(width, height) {
    if (!height && typeof width === 'object') {
      this.config.window = width;
      return this
    }

    this.config.window = { width, height };
    return this
  }

  getBrowser(name) {
    return this.getCapability('browserName')
  }

  setBrowser(name, version = '') {
    this.addCapability('browserName', name);

    if (version) {
      this.setBrowserVersion(version);
    }

    return this
  }

  setHeadless() {
    this.config.xvfb = false;
    return this
  }

  getBrowserVersion() { return undefined }

  setBrowserVersion() { return this }

  setOs(...args) { return this.setOS(...args) }

  setOsVersion(...args) { return this.setOSVersion(...args) }

  setOS() { return this }

  setOSVersion() { return this }

  setDevice() { return this }

  isReady() {
    return this.ready
  }

  _start() {}

  _close() {}

  _page() {}

  async start(capabilities, ...args) {
    await this.callHook('start:before');

    try {
      await this._start(capabilities, ...args);

      await this.callHook('start:after', this.driver);

      this.ready = true;

      onExit(() => this.close());

      return this
    /* istanbul ignore next */
    } catch (e) {
      await this.close();

      throw new browserstackLocal.BrowserError(e)
    }
  }

  async close(...args) {
    await this.callHook('close:before');

    await this._close(...args);

    await this.callHook('close:after');
  }

  async page(...args) {
    await this.callHook('page:before');

    const page = await this._page(...args);

    await this.callHook('page:after', page);

    return page
  }
}

exports.Browser = Browser;
exports.index = index;
